Module: Ocran

Defined in:
lib/ocran/cosmo_toolchain.rb,
lib/ocran.rb,
lib/ocran/option.rb,
lib/ocran/runner.rb,
lib/ocran/ed_icon.rb,
lib/ocran/version.rb,
lib/ocran/direction.rb,
lib/ocran/zip_writer.rb,
lib/ocran/dir_builder.rb,
lib/ocran/build_facade.rb,
lib/ocran/build_helper.rb,
lib/ocran/stub_builder.rb,
lib/ocran/file_path_set.rb,
lib/ocran/command_output.rb,
lib/ocran/build_constants.rb,
lib/ocran/refine_pathname.rb,
lib/ocran/library_detector.rb,
lib/ocran/rubyopt_processor.rb,
lib/ocran/gem_spec_queryable.rb,
lib/ocran/host_config_helper.rb,
lib/ocran/runtime_environment.rb,
lib/ocran/zip_payload_builder.rb,
lib/ocran/launcher_batch_builder.rb,
lib/ocran/library_detector_posix.rb,
lib/ocran/launcher_event_recorder.rb,
lib/ocran/windows_command_escaping.rb,
lib/ocran/inno_setup_script_builder.rb

Overview

NOTE: no file-scope require of digest/fileutils/tmpdir. This file is loaded while the command line is parsed, i.e. before OCRAN snapshots $LOADED_FEATURES for dependency detection (see Ocran::Option#load_cosmo_toolchain); anything required here would end up in that diff and be packed into the user's application. The three libraries are only needed at build time - after the snapshot - so they are required inside the methods that use them.

Defined Under Namespace

Modules: BuildConstants, BuildHelper, CommandOutput, CosmoToolchain, EdIcon, GemSpecQueryable, HostConfigHelper, LibraryDetector, RefinePathname, WindowsCommandEscaping, ZipWriter Classes: BuildFacade, DirBuilder, Direction, FilePathSet, InnoSetupScriptBuilder, LauncherBatchBuilder, LauncherEventRecorder, Option, RubyoptProcessor, Runner, RuntimeEnvironment, StubBuilder, ZipPayloadBuilder

Constant Summary collapse

VERSION =
"1.4.5"

Instance Attribute Summary collapse

Instance Attribute Details

#optionObject

Returns the value of attribute option.



6
7
8
# File 'lib/ocran.rb', line 6

def option
  @option
end